"""
This file contains the database models for calculations
"""

import os
import logging
logger = logging.getLogger(__name__)

from django.db.models import Model, CharField, DateTimeField, DateField, UniqueConstraint


class StationTimeline(Model):
    """
    Represents computations of sunrise, sunset of the given stations at the given timestamps.
    Day and night are derived from sunset/sunrise data.
    The day/sunrise/sunset is always on the date of the timestamp.
    The night is usually the one _starting_ on the date of the time stamp, unless the given timestamp falls
    before sunrise, in which case it is the night _ending_ on the timestamp date.
    """
    station_name = CharField(max_length=16, null=False, editable=False, help_text='The LOFAR station name.')
    timestamp = DateField(editable=False, null=True, help_text='The date (YYYYMMDD).')

    sunrise_start = DateTimeField(null=True, help_text='Start time of the sunrise.')
    sunrise_end = DateTimeField(null=True, help_text='End time of the sunrise.')
    sunset_start = DateTimeField(null=True, help_text='Start time of the sunset.')
    sunset_end = DateTimeField(null=True, help_text='End time of the sunset.')

    class Meta:
        # ensure there are no duplicate station-timestamp combinations
        constraints = [UniqueConstraint(fields=['station_name', 'timestamp'], name='unique_station_time_line')]
