"""
A viewset extended with annotation and other common generic behaviours.

Adds the following functionality:
- Swagger API annotation
"""

import logging
logger = logging.getLogger(__name__)

from rest_framework import viewsets
from drf_yasg.utils import swagger_auto_schema
from rest_framework import mixins
import json
from django.shortcuts import get_object_or_404
from django.http import JsonResponse
from django.urls import reverse as revese_url
from rest_framework.decorators import action
from lofar.common import json_utils
from lofar.sas.tmss.tmss.tmssapp.viewsets.permissions import TMSSPermissions, IsProjectMemberFilterBackend
from lofar.sas.tmss.tmss.tmssapp.models import permissions
from django_filters.rest_framework import DjangoFilterBackend
from rest_framework.filters import OrderingFilter

#class TMSSPermissionsMixin:

    # def __init__(self, *args, **kwargs):
    #     self.permission_classes = (TMSSPermissions,)
    #     self.filter_backends = (IsProjectMemberFilterBackend,)
    #     self.extra_action_permission_classes = self._create_extra_action_permission_classes()
    #     super(TMSSPermissionsMixin, self).__init__(*args, **kwargs)
    #
    # # TODO: Cache this method to avoid redundancy and overhead.
    # def _create_extra_action_permission_classes(self):
    #     extra_action_permission_classes = []
    #     extra_actions = [a.__name__ for a in self.get_extra_actions()]
    #     for ea in extra_actions:  # Create permission classes
    #         permission_name = f'{ea}_{self.serializer_class.Meta.model.__name__.lower()}'
    #         permission_class_name = f'Can {ea} {self.serializer_class.Meta.model.__name__.lower()}'
    #         new_permission_class = type(f'{permission_class_name}', (permissions.TMSSBasePermissions,), {
    #             # TODO: Is it necessary to have both permissions and object permissions?
    #             # TODO: Find a way to use the "%(app_label)s." syntax.
    #             'permission_name': permission_name,
    #             'has_permission': lambda self, request, view: request.user.has_perm(f'tmssapp.{self.permission_name}'),
    #             'has_object_permission': lambda self, request, view, obj: request.user.has_perm(f'tmssapp.{self.permission_name}'),
    #         })
    #         new_permission_class.__setattr__(self, 'permission_name', permission_name)
    #         extra_action_permission_classes.append({ea: new_permission_class},)
    #     return extra_action_permission_classes
    #
    # # TODO: Refactoring.
    # def get_model_permissions(self):
    #     extra_actions = [a.__name__ for a in self.get_extra_actions()]
    #     if self.action in extra_actions:
    #         for ea_permission_class in self.extra_action_permission_classes:
    #             if ea_permission_class.get(self.action):
    #                 return [permissions.TMSSBasePermissions, ea_permission_class.get(self.action),]
    #             else:
    #                 return [permissions.TMSSBasePermissions,]
    #     else:
    #         return [permissions.TMSSBasePermissions, ]

    #def get_permissions(self):
    #    self.get_extra_action_permission_classes()
    #    return super(TMSSPermissionsMixin, self).get_permissions()


class LOFARViewSet(viewsets.ModelViewSet):
    """
    If you're using format suffixes, make sure to also include
    the `format=None` keyword argument for each action.
    """
    permission_classes = (TMSSPermissions,)
    filter_backends = (DjangoFilterBackend, OrderingFilter, IsProjectMemberFilterBackend,)

    @swagger_auto_schema(responses={403: 'forbidden'})
    def list(self, request, **kwargs):
        return super(LOFARViewSet, self).list(request, **kwargs)

    @swagger_auto_schema(responses={400: 'invalid specification', 403: 'forbidden'})
    def create(self, request, **kwargs):
        return super(LOFARViewSet, self).create(request, **kwargs)

    @swagger_auto_schema(responses={403: 'forbidden', 404: 'not found'})
    def retrieve(self, request, pk=None, **kwargs):
        return super(LOFARViewSet, self).retrieve(request, pk, **kwargs)

    @swagger_auto_schema(responses={400: 'invalid specification', 403: 'forbidden', 404: 'not found'})
    def update(self, request, pk=None, **kwargs):
        return super(LOFARViewSet, self).update(request, pk, **kwargs)

    @swagger_auto_schema(responses={400: 'invalid specification', 403: 'forbidden', 404: 'not found'})
    def partial_update(self, request, pk=None, **kwargs):
        return super(LOFARViewSet, self).partial_update(request, pk, **kwargs)

    @swagger_auto_schema(responses={403: 'forbidden', 404: 'not found'})
    def destroy(self, request, pk=None, **kwargs):
        return super(LOFARViewSet, self).destroy(request, pk, **kwargs)

class LOFARNestedViewSet(mixins.CreateModelMixin,
                                mixins.ListModelMixin,
                                #mixins.RetrieveModelMixin,
                                viewsets.GenericViewSet):

    @swagger_auto_schema(responses={403: 'forbidden'})
    def list(self, request, **kwargs):
        return super(LOFARNestedViewSet, self).list(request, **kwargs)

    @swagger_auto_schema(responses={400: 'invalid specification', 403: 'forbidden'})
    def create(self, request, **kwargs):
        return super(LOFARNestedViewSet, self).create(request, **kwargs)



class LOFARCopyViewSet(mixins.CreateModelMixin, viewsets.GenericViewSet):

    """
    @swagger_auto_schema(responses={403: 'forbidden'})
    def list(self, request, **kwargs):
        return super(LOFARCopyViewSet, self).list(request, **kwargs)
    """

    @swagger_auto_schema(responses={400: 'invalid specification', 403: 'forbidden'})
    def create(self, request, **kwargs):
        return super(LOFARCopyViewSet, self).create(request, **kwargs)


class AbstractTemplateViewSet(LOFARViewSet):
    def retrieve(self, request, pk=None, **kwargs):
        response = super().retrieve(request, pk, **kwargs)

        # config Access-Control. Our schemas use $ref url's to other schemas, mainly pointing to our own common schemas with base definitions.
        # We instruct the client to allow fetching those.
        response["Access-Control-Allow-Origin"] = "*"
        response["Access-Control-Allow-Methods"] = "GET, OPTIONS"
        return response

    @swagger_auto_schema(responses={200: 'The schema as a JSON object',
                                    403: 'forbidden'},
                         operation_description="Get the schema as a JSON object.")
    @action(methods=['get'], detail=True)
    def schema(self, request, pk=None):
        template = get_object_or_404(self.queryset.model, pk=pk)
        return JsonResponse(template.schema, json_dumps_params={'indent': 2})

    @swagger_auto_schema(responses={200: 'The schema as a JSON object',
                                    403: 'forbidden'},
                         operation_description="Get the schema as a JSON object with all $ref URL's resolved, resulting in a complete and self describing schema.")
    @action(methods=['get'], detail=True)
    def ref_resolved_schema(self, request, pk=None):
        template = get_object_or_404(self.queryset.model, pk=pk)
        schema = json_utils.resolved_refs(template.schema)
        return JsonResponse(schema, json_dumps_params={'indent': 2})

    @swagger_auto_schema(responses={200: 'JSON object with all the defaults from the schema filled in',
                                    403: 'forbidden'},
                         operation_description="Get a JSON object with all the defaults from the schema filled in.")
    @action(methods=['get'], detail=True)
    def default(self, request, pk=None):
        template = get_object_or_404(self.queryset.model, pk=pk)
        spec = json_utils.get_default_json_object_for_schema(template.schema)
        return JsonResponse(spec, json_dumps_params={'indent': 2})


