from django.shortcuts import render, redirect
from rest_framework import viewsets, mixins, status

from rest_framework.response import Response
from rest_framework.decorators import action
from lofar.sas.tmss.tmss.workflowapp import models

from django.views import generic
from viewflow.flow.views import StartFlowMixin, FlowMixin
from viewflow.decorators import flow_start_view, flow_view
from viewflow.flow.views.utils import get_next_task_url
from django.forms import CharField, CheckboxInput
from django.forms.models import modelform_factory

from viewflow.models import Task, Process
from drf_yasg import openapi
from drf_yasg.utils import swagger_auto_schema
from drf_yasg.inspectors import SwaggerAutoSchema
from drf_yasg.openapi import Parameter
from django.core.serializers import serialize
from django.http import HttpResponse
from django.urls import NoReverseMatch

from viewflow.flow import views, viewset
from viewflow.flow.views.actions import BaseTaskActionView


from .. import forms, models, serializers, flows
import logging
logger = logging.getLogger(__name__)
import requests
from django.utils import timezone

#Viewsets and serializers to access intermediate steps of the QA Workflow
#through DRF
class QAReportingTOViewSet(viewsets.ModelViewSet):
  queryset = models.QAReportingTO.objects.all()
  serializer_class = serializers.QAReportingTOSerializer
  
class QAReportingSOSViewSet(viewsets.ModelViewSet):
  queryset = models.QAReportingSOS.objects.all()
  serializer_class = serializers.QAReportingSOSSerializer
  
class PIVerificationViewSet(viewsets.ModelViewSet):
  queryset = models.PIVerification.objects.all()
  serializer_class = serializers.PIVerificationSerializer
  
class DecideAcceptanceViewSet(viewsets.ModelViewSet):
  queryset = models.DecideAcceptance.objects.all()
  serializer_class = serializers.DecideAcceptanceSerializer

class SchedulingUnitProcessViewSet(viewsets.ModelViewSet):
  queryset = models.SchedulingUnitProcess.objects.all()
  serializer_class = serializers.SchedulingUnitProcessSerializer

class SchedulingUnitTaskViewSet(viewsets.ModelViewSet):
  queryset = Task.objects.all()
  serializer_class = serializers.SchedulingUnitTaskSerializer

class QAReportingTOView(FlowMixin, generic.CreateView):
    template_name = 'qa_reporting.html'
    model = models.QAReportingTO
    #form_class=forms.QAReportingTO
    fields = [
        'operator_report', 'operator_accept'
    ]

    def form_valid(self, form):
        report_data = form.save(commit=False)
        report_data.save()
        
        self.activation.process.qa_reporting_to = report_data
        self.activation.process.save()

        self.activation_done()
        try:
            return redirect(self.get_success_url())
        except NoReverseMatch as e:
            return

    def activation_done(self, *args, **kwargs):
        """Finish the task activation."""
        logging.info('Activation done')
        self.activation.done()

class QAReportingSOSView(FlowMixin, generic.CreateView):
    template_name = 'qa_reporting.html'
    model = models.QAReportingSOS
    fields = [
        'sos_report', 'quality_within_policy','sos_accept_show_pi'
    ]

    def form_valid(self, form):
        report_data = form.save(commit=False)
        report_data.save()
        
        self.activation.process.qa_reporting_sos = report_data
        self.activation.process.save()

        self.activation_done()
        try:
            return redirect(self.get_success_url())
        except NoReverseMatch as e:
            return

    def activation_done(self, *args, **kwargs):
        """Finish the task activation."""
        logging.info('Activation done')
        self.activation.done()


class PIVerificationView(FlowMixin, generic.CreateView):
    template_name = 'qa_reporting.html'
    model = models.PIVerification
    fields = [
        'pi_report', 'pi_accept'
    ]

    def form_valid(self, form):
        report_data = form.save(commit=False)
        report_data.save()
        
        self.activation.process.pi_verification = report_data
        self.activation.process.save()

        self.activation_done()
        try:
            return redirect(self.get_success_url())
        except NoReverseMatch as e:
            return

    def activation_done(self, *args, **kwargs):
        """Finish the task activation."""
        logging.info('Activation done')
        self.activation.done()


class DecideAcceptanceView(FlowMixin, generic.CreateView):
    template_name = 'qa_reporting.html'
    model = models.DecideAcceptance
    fields = [
        'sos_accept_after_pi'
   ]

    def form_valid(self, form):
        report_data = form.save(commit=False)
        report_data.save()
        
        self.activation.process.decide_acceptance = report_data
        self.activation.process.save()

        self.activation_done()
        try:
            return redirect(self.get_success_url())
        except NoReverseMatch as e:
            return

    def activation_done(self, *args, **kwargs):
        """Finish the task activation."""
        logging.info('Activation done')
        self.activation.done()

class UnpinDataView(FlowMixin, generic.CreateView):
    template_name = 'qa_reporting.html'
    
    model = models.UnpinData
    fields = [
        'unpin_data'
   ]

    def form_valid(self, form):
        report_data = form.save(commit=False)
        report_data.save()
        
        self.activation.process.unpin_data = report_data
        self.activation.process.save()        
        self.activation_done()
        try:
            return redirect(self.get_success_url())
        except NoReverseMatch as e:
            return

    def activation_done(self, *args, **kwargs):
        # TODO: Should Wait for data to be unpinned?  
        """Finish the task activation."""
        logging.info('Activation done')
        self.activation.done()



class SchedulingUnitTaskAssignViewSet(mixins.CreateModelMixin,
                                #mixins.ListModelMixin,
                                #mixins.RetrieveModelMixin,
                                viewsets.GenericViewSet):
  queryset = Task.objects.all()
  serializer_class = serializers.SchedulingUnitAssignTaskSerializer

  @swagger_auto_schema(responses={200: 'Assign Scheduling Unit Task to the specified user',
                                    403: 'forbidden',
                                    422: 'error'},
                         operation_description="Assign a Scheduling Unit Task to an user")
  def create(self, request, *args, **kwargs):
    if 'qa_scheduling_unit_task_id' in kwargs:
      try:
        Task.objects.filter(id=self.kwargs['qa_scheduling_unit_task_id'])[0].activate().assign(self.request.user)
        content = {'Assigned': 'Scheduling Unit Task assigned to the specified user'}
        return Response(content, status=status.HTTP_200_OK)
      except AttributeError:
        content = {'AttributeError': 'Cannot assign the specified Scheduling Unit Task to an user'}
        return Response(content, status=status.HTTP_422_UNPROCESSABLE_ENTITY)
      except IndexError:
        content = {'IndexError': 'No Scheduling Unit Task with the specified id'}
        return Response(content, status=status.HTTP_422_UNPROCESSABLE_ENTITY)

 
class SchedulingUnitTaskUnassignViewSet(mixins.CreateModelMixin,
                                #mixins.ListModelMixin,
                                #mixins.RetrieveModelMixin,
                                viewsets.GenericViewSet):
  queryset = Task.objects.all()
  serializer_class = serializers.SchedulingUnitUnassignTaskSerializer

  @swagger_auto_schema(responses={200: '',
                                    403: 'forbidden',
                                    422: 'error'},
                         operation_description="Unassign a Scheduling Unit Task")
  def create(self, request, *args, **kwargs):
    if 'qa_scheduling_unit_task_id' in kwargs:
      try:
        Task.objects.filter(id=self.kwargs['qa_scheduling_unit_task_id'])[0].activate().unassign()
        content = {'Unassign': 'Scheduling Unit Task unassigned'}
        return Response(content, status=status.HTTP_200_OK)
      except AttributeError:
        content = {'Unassign': 'Cannot unassign the specified Scheduling Unit Task'}
        return Response(content, status=status.HTTP_422_UNPROCESSABLE_ENTITY)
      except IndexError:
        content = {'Unassign': 'No Scheduling Unit Task with the specified id'}
        return Response(content, status=status.HTTP_422_UNPROCESSABLE_ENTITY)


class SchedulingUnitGetActiveTasksViewSet(mixins.CreateModelMixin,
                                #mixins.ListModelMixin,
                                #mixins.RetrieveModelMixin,
                                viewsets.GenericViewSet):
  
  queryset = models.SchedulingUnitProcess.objects.all()
  serializer_class = serializers.SchedulingUnitGetActiveTasksSerializer
  
  @swagger_auto_schema(responses={200: 'List of non finished tasks.',
                                    403: 'forbidden',
                                    422: 'error'},
                         operation_description="Get the list of active tasks.")
  def create(self, request, *args, **kwargs):
    if 'qa_scheduling_unit_process_id' in kwargs:
      try:
        data = serialize('json', models.SchedulingUnitProcess.objects.filter(id=self.kwargs['qa_scheduling_unit_process_id'])[0].active_tasks())
        return HttpResponse(data, content_type='application/json')
      except IndexError:
        content = {'Get Active Task(s)': 'No Process with the specified id'}
        return Response(content, status=status.HTTP_422_UNPROCESSABLE_ENTITY)

 
        

class SchedulingUnitTaskExecuteViewSet(mixins.CreateModelMixin,
                                #mixins.ListModelMixin,
                                #mixins.RetrieveModelMixin,
                                viewsets.GenericViewSet):
  
  queryset = models.SchedulingUnitProcess.objects.all()
  serializer_class = serializers.SchedulingUnitGetActiveTasksSerializer

  @swagger_auto_schema(responses={200: '',
                                    403: 'forbidden',
                                    422: 'error'},
                         operation_description="Unassign a Scheduling Unit Task")

  def create(self, request, *args, **kwargs):
    if 'qa_scheduling_unit_process_id' in kwargs:

      try:
        process= models.SchedulingUnitProcess.objects.get(pk=self.kwargs['qa_scheduling_unit_process_id'])
        task = models.SchedulingUnitProcess.objects.get(pk=self.kwargs['qa_scheduling_unit_process_id']).active_tasks()[0]
        view =  task.flow_task._view_class.as_view()

        act=task.activate()
        act.prepare()

        # Prepare the POST request's fields
        request._request.POST = request._request.POST.copy()
        for field in request.data:
            request._request.POST[field] = request.data[field]
        request._request.POST['_viewflow_activation-started'] = timezone.now()
        request._request.POST['_done'] = ''

        response = view(request._request, flow_class=flows.SchedulingUnitFlow, flow_task=task.flow_task,
        process_pk=process.pk, task_pk=task.pk)

        content = {'Perform Task': 'Task Performed'}
        return Response(content, status=status.HTTP_200_OK)
      
      except AttributeError:
        content = {'Perform Task': 'Cannot perform the active Scheduling Unit Task'}
        return Response(content, status=status.HTTP_422_UNPROCESSABLE_ENTITY)
      
      except IndexError:
        content = {'Perform Task': 'No Scheduling Unit Process with the specified id'}
        return Response(content, status=status.HTTP_422_UNPROCESSABLE_ENTITY)
